--
-- AdditionalSettingsGui
--
-- @author Rockstar
-- @fs25 16/12/2024
--


AdditionalSettingsGui = {}

local AdditionalSettingsGui_mt = Class(AdditionalSettingsGui)

function AdditionalSettingsGui.new(baseDir, custom_mt)
	local self = setmetatable({}, custom_mt or AdditionalSettingsGui_mt)

	self.baseDirectory = baseDir
	self.settingsFrame = g_inGameMenu.pageSettings
	self.saveFrame = g_inGameMenu.pageSave
	self.screenController = nil

	AdditionalSettingsUtil.appendedFunction(self.settingsFrame, "onFrameOpen", self, "settings_onFrameOpen")
	AdditionalSettingsUtil.prependedFunction(self.settingsFrame, "onFrameClose", self, "settings_onFrameClose")
	AdditionalSettingsUtil.overwrittenFunction(self.settingsFrame.subCategoryPaging, "onClickCallback", self, "settings_onClickCallback")
	AdditionalSettingsUtil.overwrittenFunction(self.settingsFrame, "inputEvent", self, "settings_inputEvent")
	AdditionalSettingsUtil.appendedFunction(self.saveFrame, "onFrameOpen", self, "save_onFrameOpen")
	AdditionalSettingsUtil.overwrittenFunction(self.saveFrame.gameSettingsButton, "onClickCallback", self, "save_onButtonGameSettings")

	return self
end

function AdditionalSettingsGui:addPage(addButtons)
	local currentGui = FocusManager.currentGui
	local screenController = AdditionalSettingsPage.register()
	local additionalSettingsPage = screenController.additionalSettingsPage
	local additionalSettingsTab = screenController.additionalSettingsTab
	local settingsFrame = self.settingsFrame

	self:addElementAtPosition(additionalSettingsPage, settingsFrame.subCategoryPages[1].parent, 3)
	self:addElementAtPosition(additionalSettingsTab, settingsFrame.subCategoryBox, 3)

	settingsFrame:updateAbsolutePosition()

	local getDescendants = settingsFrame.getDescendants

	settingsFrame.getDescendants = function()
		return additionalSettingsPage:getDescendants()
	end

	settingsFrame:exposeControlsAsFields(settingsFrame.name)
	settingsFrame.getDescendants = getDescendants

	additionalSettingsPage:setTarget(settingsFrame, additionalSettingsPage.target)
	additionalSettingsTab:setTarget(settingsFrame, additionalSettingsTab.target)

	FocusManager:setGui(settingsFrame.name)
	FocusManager:removeElement(additionalSettingsPage)
	FocusManager:removeElement(additionalSettingsTab)
	FocusManager:loadElementFromCustomValues(additionalSettingsPage)
	FocusManager:loadElementFromCustomValues(additionalSettingsTab)

	local subCategoryPages = settingsFrame.subCategoryPages

	local gamePage = subCategoryPages[1]
	local generalPage = subCategoryPages[2]
	local additionalPage = additionalSettingsPage
	local graphicsPage = subCategoryPages[3]
	local serverPage = subCategoryPages[4]
	local controlsPage = subCategoryPages[5]

	settingsFrame.subCategoryPages = {
		[1] = gamePage,
		[2] = generalPage,
		[3] = additionalPage,
		[4] = graphicsPage,
		[5] = serverPage,
		[6] = controlsPage
	}

	local subCategoryTabs = settingsFrame.subCategoryTabs

	local gameTab = subCategoryTabs[1]
	local generalTab = subCategoryTabs[2]
	local additionalTab = additionalSettingsTab
	local graphicsTab = subCategoryTabs[3]
	local serverTab = subCategoryTabs[4]
	local controlsTab = subCategoryTabs[5]

	settingsFrame.subCategoryTabs = {
		[1] = gameTab,
		[2] = generalTab,
		[3] = additionalTab,
		[4] = graphicsTab,
		[5] = serverTab,
		[6] = controlsTab
	}

	InGameMenuSettingsFrame.SUB_CATEGORY.GAME_SETTINGS = 1
	InGameMenuSettingsFrame.SUB_CATEGORY.GENERAL_SETTINGS = 2
	InGameMenuSettingsFrame.SUB_CATEGORY.ADDITIONAL_SETTINGS = 3
	InGameMenuSettingsFrame.SUB_CATEGORY.GRAPHIC_SETTINGS = 4
	InGameMenuSettingsFrame.SUB_CATEGORY.SERVER_SETTINGS = 5
	InGameMenuSettingsFrame.SUB_CATEGORY.CONTROLS = 6

	InGameMenuSettingsFrame.HEADER_SLICES = {
		[InGameMenuSettingsFrame.SUB_CATEGORY.GAME_SETTINGS] = "gui.icon_options_gameSettings2",
		[InGameMenuSettingsFrame.SUB_CATEGORY.GENERAL_SETTINGS] = "gui.icon_options_generalSettings2",
		[InGameMenuSettingsFrame.SUB_CATEGORY.ADDITIONAL_SETTINGS] = "gui.icon_options_device",
		[InGameMenuSettingsFrame.SUB_CATEGORY.GRAPHIC_SETTINGS] = "gui.icon_options_displaySettings",
		[InGameMenuSettingsFrame.SUB_CATEGORY.SERVER_SETTINGS] = "gui.icon_options_serverSettings",
		[InGameMenuSettingsFrame.SUB_CATEGORY.CONTROLS] = "gui.icon_options_keyboardControls"
	}

	InGameMenuSettingsFrame.HEADER_TITLES = {
		[InGameMenuSettingsFrame.SUB_CATEGORY.GAME_SETTINGS] = "ui_ingameMenuGameSettingsGame",
		[InGameMenuSettingsFrame.SUB_CATEGORY.GENERAL_SETTINGS] = "ui_ingameMenuGameSettingsGeneral",
		[InGameMenuSettingsFrame.SUB_CATEGORY.ADDITIONAL_SETTINGS] = "ags_ui_ingameMenuAdditionalSettings",
		[InGameMenuSettingsFrame.SUB_CATEGORY.GRAPHIC_SETTINGS] = "ui_inGameMenuGraphicSettings",
		[InGameMenuSettingsFrame.SUB_CATEGORY.SERVER_SETTINGS] = "button_serverSettings",
		[InGameMenuSettingsFrame.SUB_CATEGORY.CONTROLS] = "ui_inGameMenuControls"
	}

	self.screenController = screenController

	if addButtons then
		local saveFrame = self.saveFrame
		local buttonsLayout = saveFrame.buttonsLayout
		local gameSettingsButton = saveFrame.gameSettingsButton

		FocusManager:setGui(saveFrame.name)

		gameSettingsButton.elements[2]:setImageSlice(nil, "gui.icon_options_gameSettings")
		gameSettingsButton.elements[3]:setText(g_i18n:getText("ui_ingameMenuGameSettingsGame"))

		local generalSettingsButton = gameSettingsButton:clone()

		generalSettingsButton.id = "generalSettingsButton"
		generalSettingsButton.elements[2]:setImageSlice(nil, "gui.icon_options_generalSettings")
		generalSettingsButton.elements[3]:setText(g_i18n:getText("ui_ingameMenuGameSettingsGeneral"))
		generalSettingsButton.onClickCallback = function()
			g_inGameMenu:openGeneralSettingsScreen()
		end

		self:addElementAtPosition(generalSettingsButton, buttonsLayout, table.find(buttonsLayout.elements, gameSettingsButton) + 1)
		generalSettingsButton:reloadFocusHandling(true)
		saveFrame.generalSettingsButton = generalSettingsButton

		local additionalSettingsButton = gameSettingsButton:clone()

		additionalSettingsButton.id = "additionalSettingsButton"
		additionalSettingsButton.elements[2]:setImageFilename(self.baseDirectory .. "gui/ui_elements.png")
		additionalSettingsButton.elements[2]:setImageUVs(nil, unpack(GuiUtils.getUVs("258px 2px 135px 135px", {512, 512})))
		additionalSettingsButton.elements[3]:setText(g_i18n:getText("ags_ui_ingameMenuAdditionalSettings"))
		additionalSettingsButton.onClickCallback = function()
			g_inGameMenu:setPageEnabled(gEnv.ClassUtil.getClassObjectByObject(settingsFrame), true)

			local index = g_inGameMenu.pagingElement:getPageMappingIndexByElement(settingsFrame)

			g_inGameMenu.pageSelector:setState(index, true)
			settingsFrame.isOpening = true
			screenController:onClickAdditionalSettings()
			settingsFrame.isOpening = false
		end

		self:addElementAtPosition(additionalSettingsButton, buttonsLayout, table.find(buttonsLayout.elements, generalSettingsButton) + 1)
		additionalSettingsButton:reloadFocusHandling(true)
		saveFrame.additionalSettingsButton = additionalSettingsButton

		local serverSettingsButton = gameSettingsButton:clone()

		serverSettingsButton.id = "serverSettingsButton"
		serverSettingsButton.elements[2]:setImageSlice(nil, "gui.icon_multiplayer_rentDedi")
		serverSettingsButton.elements[3]:setText(g_i18n:getText("button_serverSettings"))
		serverSettingsButton.onClickCallback = function()
			g_inGameMenu:setPageEnabled(gEnv.ClassUtil.getClassObjectByObject(settingsFrame), true)

			local index = g_inGameMenu.pagingElement:getPageMappingIndexByElement(settingsFrame)

			g_inGameMenu.pageSelector:setState(index, true)
			settingsFrame.isOpening = true
			settingsFrame:onClickServerSettings()
			settingsFrame.isOpening = false
		end

		self:addElementAtPosition(serverSettingsButton, buttonsLayout, table.find(buttonsLayout.elements, additionalSettingsButton) + 1)
		serverSettingsButton:reloadFocusHandling(true)
		saveFrame.serverSettingsButton = serverSettingsButton

		for _, element in ipairs(buttonsLayout.elements) do
			if element:isa(ButtonElement) and element.profile == "fs25_saveButton" then
				for _, elem in ipairs(element.elements) do
					if elem:isa(TextElement) and elem.profile == "fs25_saveButtonText" then
						elem.sizeStr = "180px 40px"
						elem:resolveSizeString()
						elem:fixThinLines()
						elem:updateAbsolutePosition()
					end
				end

				element.widthStr = "220px"
				element:resolveSizeString()
				element:fixThinLines()
				element:updateAbsolutePosition()
			end
		end

		buttonsLayout:invalidateLayout()
	end

	FocusManager:setGui(currentGui)

	return screenController
end

function AdditionalSettingsGui:addElementAtPosition(element, target, position)
	if element.parent ~= nil then
		element.parent:removeElement(element)
	end

	table.insert(target.elements, position, element)

	element.parent = target
end

function AdditionalSettingsGui:settings_onFrameOpen(settingsFrame)
	settingsFrame.isOpening = true

	if self.screenController ~= nil then
		self.screenController:onFrameOpen()
	end

	settingsFrame.isOpening = false
end

function AdditionalSettingsGui:settings_onFrameClose(settingsFrame)
	if self.screenController ~= nil then
		self.screenController:onFrameClose()
	end
end

function AdditionalSettingsGui:save_onFrameOpen(saveFrame)
	if saveFrame.serverSettingsButton ~= nil then
		saveFrame.serverSettingsButton:setVisible(saveFrame.hasMasterRights and g_currentMission.missionDynamicInfo.isMultiplayer)
		saveFrame.buttonsLayout:invalidateLayout()
	end
end

function AdditionalSettingsGui:save_onButtonGameSettings(saveFrame, superFunc)
	local subCategoryPaging = self.settingsFrame.subCategoryPaging
	local state = subCategoryPaging.state

	superFunc(saveFrame)
	subCategoryPaging:setState(state, true)
end

function AdditionalSettingsGui:settings_onClickCallback(settingsFrame, superFunc, state)
	local retValue = superFunc(settingsFrame, state)
	local value = settingsFrame.subCategoryPaging.texts[state]

	if value ~= nil and tonumber(value) == InGameMenuSettingsFrame.SUB_CATEGORY.ADDITIONAL_SETTINGS then
		if self.screenController ~= nil then
			self.screenController:onTabOpen()
		end

		local additionalSettingsLayout = settingsFrame.additionalSettingsLayout

		settingsFrame.settingsSlider:setDataElement(additionalSettingsLayout)
		FocusManager:linkElements(settingsFrame.subCategoryPaging, FocusManager.TOP, additionalSettingsLayout.elements[#additionalSettingsLayout.elements].elements[1])
		FocusManager:linkElements(settingsFrame.subCategoryPaging, FocusManager.BOTTOM, additionalSettingsLayout:findFirstFocusable(true))
	end

	return retValue
end

function AdditionalSettingsGui:settings_inputEvent(settingsFrame, superFunc, action, value, eventUsed)
	local retValue = superFunc(settingsFrame, action, value, eventUsed)
	local pressedAccept = false
	local element = FocusManager.currentFocusData.focusElement

	if element ~= nil and not element.needExternalClick then
		pressedAccept = action == InputAction.MENU_ACCEPT

		if pressedAccept and not FocusManager:isFocusInputLocked(action) and element:getIsFocused() and element:getIsVisible() then
			FocusManager.focusSystemMadeChanges = true
			element:onFocusActivate()
			FocusManager.focusSystemMadeChanges = false
		end
	end

	return retValue or pressedAccept
end